let NOAB = window.NOAB || {};
(function ()
{
	const statecodeFilter =
	{
		"column": "statecode",
		"operator": "eq",
		"value": 0
	};
	
	let map, markers, markerIcon;
	
	function loadMap()
	{
		map = L.map('map').setView([52.2129919, 5.2793703], 7);
		L.tileLayer('https://tile.openstreetmap.org/{z}/{x}/{y}.png', {
			maxZoom: 19,
			attribution: '&copy; <a href="http://www.openstreetmap.org/copyright">OpenStreetMap</a>'
		}).addTo(map);
		
		markers = {};
		
		markerIcon = L.icon({
			iconUrl: '/noab-marker-account.png',
			iconSize: [15, 23],
			iconAnchor: [7, 23],
			popupAnchor: [0, -31]
		});
	}
	this.LoadMap = loadMap;

	function filterAccounts()
	{
		let fetchXml = `
		<fetch distinct="true">
		  <entity name="account">
			<attribute name="name" />
			<attribute name="address1_city" />
			<attribute name="accountid" />
			<attribute name="address1_latitude" />
			<attribute name="address1_longitude" />
			<attribute name="address1_line1" />
			<attribute name="address1_line2" />
			<attribute name="address1_line3" />
			<attribute name="address1_postalcode" />
			<filter type="and">
			  <condition attribute="statecode" operator="eq" value="0" />
			  <condition attribute="name" operator="not-null" />
			  ${getSimpleFilter("filter_Lidmaatschapsstatus", "noab_membership_status", [2,3])}
			  ${getSimpleFilter("filter_Plaats", "address1_city")}
			  ${getNameFilter("name")}
			  ${getAccountSizeFilter()}
			</filter>
			${getBranchesFilter()}
			${getExpertisesFilter()}
			${getAfdelingFilter()}
			${getSoftwareFilterAccounts()}
			${getBeschikbaarVoorFilter()}
			<order attribute="name" />
		  </entity>
		</fetch>
	  `;

		filter(
		{
			setName: "accounts",
			id: "entitylistrows",
			primaryid: "accountid",
			latitude: "address1_latitude",
			longitude: "address1_longitude",
			details: "/community/kantoor?id=",
			name: "name",
			street: "address1_line1",
			housenumber: "address1_line2",
			housenumberAddition: "address1_line3",
			postalcode: "address1_postalcode",
			city: "address1_city",
			columns:
			[
				{
					name: "name",
					link: "/community/kantoor?id="
				},
				{
					name: "address1_city"
				}
			]
		}, fetchXml);
	}
	this.FilterAccounts = filterAccounts;
	
	function filterAccountsAnonymous()
	{
		let fetchXml = `
		<fetch distinct="true">
		  <entity name="noab_members">
			<attribute name="noab_name" />
			<attribute name="noab_city" />
			<attribute name="noab_membersid" />
			<attribute name="noab_latitude" />
			<attribute name="noab_longitude" />
			<attribute name="noab_street" />
			<attribute name="noab_housenumber" />
			<attribute name="noab_housenumberaddition" />
			<attribute name="noab_postalcode" />
			<filter type="and">
			  <condition attribute="statecode" operator="eq" value="0" />
			  <condition attribute="noab_name" operator="not-null" />
			  ${getSimpleFilter("filter_Plaats", "noab_city")}
			  ${getNameFilter("noab_name")}
			</filter>
			<order attribute="noab_name" />
		  </entity>
		</fetch>`;

		filter(
		{
			setName: "noab_memberses",
			id: "entitylistrows",
			primaryid: "noab_membersid",
			latitude: "noab_latitude",
			longitude: "noab_longitude",
			details: "/kantoren/kantoor?id=",
			name: "noab_name",
			street: "noab_street",
			housenumber: "noab_housenumber",
			housenumberAddition: "noab_housenumberaddition",
			postalcode: "noab_postalcode",
			city: "noab_city",
			columns:
			[
				{
					name: "noab_name",
					link: "/kantoren/kantoor?id="
				},
				{
					name: "noab_city"
				}
			]
		}, fetchXml);
	}
	this.FilterAccountsAnonymous = filterAccountsAnonymous;

	function filterPersons()
	{
		let fetchXml = `
			<fetch distinct="true">
			  <entity name="pyl_person">
				<attribute name="pyl_fullname" />
				<attribute name="pyl_personid" />
				<filter type="and">
				  <condition attribute="noab_visible_profile" operator="ne" value="0" />
				  <condition attribute="statecode" operator="eq" value="0" />
				  <condition attribute="pyl_fullname" operator="not-null" />
				  ${getNameFilter("pyl_fullname")}
				</filter>
				${getSoftwareFilterPeronen()}
				${getVaardighedenFilter()}
				${getVerenigingenFilter()}
				${getBeschikbaarVoorFilter_Person()}
				<order attribute="pyl_fullname" />
				<link-entity name="contact" from="pyl_person" to="pyl_personid">
				  <filter>
					<condition attribute="statecode" operator="eq" value="0" />
				    <condition attribute="noab_office_login" operator="eq" value="0" />
				  </filter>
				  <link-entity name="account" from="accountid" to="parentcustomerid" alias="a">
					<attribute name="accountid" />
					<attribute name="name" />
					<filter>
					  <condition attribute="statecode" operator="eq" value="0" />
					</filter>
				  </link-entity>
				</link-entity>
			  </entity>
			</fetch>
		`;

		filter(
		{
			setName: "pyl_persons",
			id: "entitylistrows",
			primaryid: "pyl_personid",
			columns:
			[
				{
					name: "pyl_fullname",
					link: "/community/profiel?id="
				},
				{
					f: concatAccounts
				}
			]
		}, fetchXml);
	}
	this.FilterPersons = filterPersons;

	function filter(tableData, fetchXml)
	{
		fetchXml = minimizeXml(fetchXml);
		
		//console.debug(fetchXml);
		
		const url = `/_api/${tableData.setName}?fetchXml=${encodeURIComponent(fetchXml)}`;
		
		$.ajax(
		{
			type: "GET",
			contentType: "application/json; charset=utf-8",
			datatype: "json",
			url: url,
			beforeSend: function (XMLHttpRequest)
			{
				XMLHttpRequest.setRequestHeader("Accept", "application/json");
				XMLHttpRequest.setRequestHeader("OData-MaxVersion", "4.0");
				XMLHttpRequest.setRequestHeader("OData-Version", "4.0");
				XMLHttpRequest.setRequestHeader("Prefer", "odata.include-annotations=\"*\"");
			},
			async: true,
			success: (data) => populate(tableData, data.value),
			error: function (xhr, textStatus, errorThrown)
			{
				console.log(errorThrown);
			}
		}
		);
	}
	
	function getNameFilter(column)
	{
		const name = $(`#filter_Name`)[0].value;
		if (name && name.length > 0)
			return addCondition(
			{
				"column": column,
				"operator": "like",
				"value": `%${name}%`
			});
		return "";
	}

	function getSoftwareFilterPeronen()
	{
		const condition = getCondition("noab_name", "filter_Softwarekennis");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_software_experience",
			"from": "noab_person",
			"to": "pyl_personid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_software",
					"from": "noab_softwareid",
					"to": "noab_software",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}
	
	function getSoftwareFilterAccounts()
	{
		const condition = getCondition("noab_name", "filter_Software");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_software_usage",
			"from": "noab_account",
			"to": "accountid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_software",
					"from": "noab_softwareid",
					"to": "noab_software",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getVaardighedenFilter()
	{
		const condition = getCondition("noab_name", "filter_Vaardigheden");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_specialty",
			"from": "noab_person",
			"to": "pyl_personid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_subject",
					"from": "noab_subjectid",
					"to": "noab_subject",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getVerenigingenFilter()
	{
		const condition = getCondition("noab_name", "filter_Verenigingen");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_membership",
			"from": "noab_person",
			"to": "pyl_personid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_association",
					"from": "noab_associationid",
					"to": "noab_association",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getBeschikbaarVoorFilter_Person()
	{
		const condition = getCondition("noab_name", "filter_Beschikbaarvoor");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_services",
			"from": "noab_person",
			"to": "pyl_personid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_businessactivities",
					"from": "noab_businessactivitiesid",
					"to": "noab_businessactivity",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getBranchesFilter()
	{
		const condition = getCondition("noab_name", "filter_Branches");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_targetaudience",
			"from": "noab_account",
			"to": "accountid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_branch",
					"from": "noab_branchid",
					"to": "noab_industry",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getExpertisesFilter()
	{
		const condition = getCondition("noab_name", "filter_Expertises");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_expertise",
			"from": "noab_expert",
			"to": "accountid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_subject",
					"from": "noab_subjectid",
					"to": "noab_subject",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getAfdelingFilter()
	{
		const condition = getCondition("pyl_gremiumname", "filter_Regio");
		return !condition ? "" : addLinkEntity(
		{
			"table": "pyl_gremium",
			"from": "pyl_gremiumid",
			"to": "noab_department",
			"conditions": [statecodeFilter, condition]
		}
		);
	}

	function getBeschikbaarVoorFilter()
	{
		const condition = getCondition("noab_name", "filter_Beschikbaarvoor");
		return !condition ? "" : addLinkEntity(
		{
			"table": "noab_services",
			"from": "noab_provider",
			"to": "accountid",
			"conditions": [statecodeFilter],
			"linkEntities":
			[
				{
					"table": "noab_businessactivities",
					"from": "noab_businessactivitiesid",
					"to": "noab_businessactivity",
					"conditions": [statecodeFilter, condition]
				}
			]
		}
		);
	}

	function getAccountSizeFilter()
	{
		let filter = "";
		
		const options = $(`#filter_Omvang`)[0].selectedOptions;
		if (options && options.length > 0)
		{
			filter = `<filter type="or">`;
			
			for (const option of options)
			{
				let condition;
				if (option.innerText.includes("-"))
				{
					condition = `<condition attribute="noab_size_people" operator="between">`;
					const numbers = option.innerText.split("-");
					for (const number of numbers)
						condition += `<value>${number}</value>`;
					condition += `</condition>`;
				}
				else // 26 of meer
					condition = `<condition attribute="noab_size_people" operator="ge" value="26"></condition>`;
				
				filter += condition;
			}
		
			filter += `</filter>`;
		}
		
		return filter;
	}

	function getSimpleFilter(filterID, column, defaultFilter)
	{
		const condition = getCondition(column, filterID, defaultFilter);
		return condition ? addCondition(condition) : "";
	}

	function addLinkEntity(entity)
	{
		let filter =
`<link-entity name="${entity.table}" from="${entity.from}" to="${entity.to}" link-type="inner">`;

		if (entity.conditions)
		{
			filter += `<filter type="and">`;
			for (const condition of entity.conditions)
				filter += addCondition(condition);
			filter += `</filter>`;
		}

		if (entity.linkEntities)
			for (const linkEntity of entity.linkEntities)
				filter += addLinkEntity(linkEntity);

		filter += `</link-entity>`;
		return filter;
	}

	function addCondition(condition)
	{
		let filter = `<condition attribute="${condition.column}" operator="${condition.operator}"`;
		if (!Array.isArray(condition.value))
			filter += ` value="${condition.value}"`;
		filter += `>`;
		if (Array.isArray(condition.value))
			for (const value of condition.value)
				filter += `<value>${value}</value>`;
		filter += `</condition>`;
		return filter;
	}

	function getCondition(column, filterID, defaultFilter)
	{
		let values = getValues(filterID);
		let condition = 
		{
			"column": column,
			"operator": "in"
		};
		if (values.length > 0)
			condition.value = values;
		else if (defaultFilter)
			condition.value = defaultFilter;
		else 
			return null;
		
		return condition;
	}

	function getValues(filterID)
	{
		let values = [];
		const options = $(`#${filterID}`)[0].selectedOptions;
		if (options && options.length > 0)
			for (const option of options)
				values.push(option.value);
		return values;
	}

	function populate(tableData, rows)
	{
		const container = $(`#${tableData.id}`)[0];
		container.innerHTML = "";
	
		const persons = new Map();
		for (const row of rows)
			persons.set(row[tableData.primaryid], row);
	
		for (const row of persons.values())
		{
			const tr = document.createElement("tr");
			tr.addEventListener("mouseenter", () =>
			{
				for (const id in markers)
					markers[id].closePopup();
				togglePopup(row[tableData.name], true);
			});
			tr.addEventListener("mouseleave", () =>
			{
				togglePopup(row[tableData.name], false);
			});
			container.appendChild(tr);

			for (const column of tableData.columns)
			{
				const td = document.createElement("td");

				if (column.f)
					column.f(tableData, rows, row).then((html) => td.appendChild(html));
				else
				{
					const value = row[column.name];
					if (value)
					{
						if (column.link)
						{
							const a = document.createElement("a");
							a.href = column.link + row[tableData.primaryid];
							a.innerText = value;
							td.appendChild(a);
						}
						else
							td.innerText = value;
					}
				}
				tr.appendChild(td);
			}
		}
		
		populateMarkers(tableData, rows);
	}
	
	function populateMarkers(tableData, rows)
	{
		if (!map)
			return;
		
		for (const id in markers)
			markers[id].setOpacity(0);
		
		let minLat = 90;
		let minLong = 180;
		let maxLat = -90;
		let maxLong = -180;
		
		for (const row of rows)
		{
			if (!markers[row[tableData.name]])
			{			
				const latitude = row[tableData.latitude];
				const longitude = row[tableData.longitude];
				if (latitude && longitude)
				{	
					if (latitude < minLat)
						minLat = latitude;
					if (latitude > maxLat)
						maxLat = latitude;
					if (longitude < minLong)
						minLong = longitude;
					if (longitude > maxLong)
						maxLong = longitude;
			
					var marker = L.marker([latitude, longitude], { icon: markerIcon }).addTo(map);
					marker.bindPopup(
						`<div class="noab-leaflet-popup-content">
							<h4>${row[tableData.name]}</h4>
							<p>${row[tableData.street]} ${row[tableData.housenumber]}${row[tableData.housenumberAddition] ?? ""}<br>${row[tableData.postalcode]} ${row[tableData.city]}</p>
							<p>
								<a href="${tableData.details}${row[tableData.primaryid]}">Bekijk profiel</a>
							</p>
						</div>`
						,
						{ className: "noab-leaflet-popup-content" }
					);
					markers[row[tableData.name]] = marker;
				}
			}
			else
			{
				markers[row[tableData.name]].setOpacity(1);
				const latLong = markers[row[tableData.name]].getLatLng();
				if (latLong.lat < minLat)
					minLat = latLong.lat;
				if (latLong.lat > maxLat)
					maxLat = latLong.lat;
				if (latLong.lng < minLong)
					minLong = latLong.lng;
				if (latLong.lng > maxLong)
					maxLong = latLong.lng;
			}
		}
		
		map.fitBounds([[minLat, minLong], [maxLat, maxLong]]);
	}
	
	function togglePopup(accountName, openPopup)
	{
		if (!map)
			return;
		
		const marker = markers[accountName];
		if (!marker)
			return;
		
		if (openPopup)
			marker.openPopup();
		else
			marker.closePopup();
	}
	this.TogglePopup = togglePopup;

	async function concatAccounts(tableData, rows, row)
	{
		const personid = row[tableData.primaryid];
			
		const accounts = [];
		rows.filter((r) => r[tableData.primaryid] == personid).forEach((r) => accounts.push({ accountid: r["a.accountid"], name: r["a.name"] }));
		accounts.sort((a,b) => 
		{
			if (a.name < b.name)
				return -1;
			if (a.name > b.name)
				return 1;
			return 0;
			
		});
		
		const div = document.createElement("div");
		for (let i = 0; i < accounts.length; i++)
		{
			const account = accounts[i];
			const a = document.createElement("a");
			a.href = `/community/kantoor?id=${account.accountid}`;
			a.innerText = account.name;
			div.appendChild(a);

			if (i < accounts.length - 1)
			{
				const span = document.createElement("span");
				span.innerText = " • ";
				div.appendChild(span);
			}
		}

		return div;
	}
	
	function minimizeXml(xml)
	{
		let dom = parseXml(xml);
		removeWhitespace(dom);
		return serializeXml(dom);
	}

	function parseXml(txt)
	{
		let xmlDoc;

		if (window.DOMParser)
		{
			const parser = new DOMParser();
			xmlDoc = parser.parseFromString(txt, "text/xml");
		}
		else // Internet Explorer
		{
			xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
			xmlDoc.async = false;
			xmlDoc.loadXML(txt);
		}

		return xmlDoc;
	}

	function serializeXml(node)
	{
		try
		{
			// XMLSerializer exists in certain browsers
			var serializer = new XMLSerializer();
			return serializer.serializeToString(node);
		}
		catch (e)
		{
			// Internet Explorer has a different approach to serializing XML
			return elem.xml;
		}

	}

	function removeWhitespace(node)
	{
		if (node.childNodes && node.childNodes.length)
			Array.prototype.slice.call(node.childNodes).forEach(removeWhitespace);
		if ((node.nodeType === 3 || node.nodeType === 4) && /^[ \r\n\t]*$/.test(node.textContent))
			node.parentNode.removeChild(node);
	}

}
).call(NOAB);
